'''
Created on 26 mars 2018

@author: Tonneau Stephane
'''
import os
import psutil
from .base_utils import first_not_null
from .base_utils import Configuration
from .base_utils import SystemEnvironment

LOCAL_DRIVER_PTYPE = "local_driver"
LOCAL_EXECUTER_PTYPE = "local_executer"


class LocalConfiguration(Configuration):
    '''Class that encapsulate a local Configuration'''

    def __init__(self, driver_memory=None, executer_memory=None, num_executers=None,
                 queue=None, executor_environment=None):

        # parent initialisation
        super(LocalConfiguration, self).__init__(driver_memory, executer_memory, num_executers,
                                                 queue, executor_environment)

    def complete(self):
        '''Fix the local configuration information'''

        self.driver_memory = first_not_null(self.driver_memory, "0")
        self.executer_memory = first_not_null(self.executer_memory, "999")
        #self.spark_master = first_not_null(self.spark_master, "LOCAL")
        #self.deploy_mode = first_not_null(self.deploy_mode, "LOCAL")
        self.min_executors = first_not_null(self.min_executors, "0")
        self.max_executors = first_not_null(self.max_executors, "999999999")
        self.num_executers = first_not_null(self.num_executers, "1")
        self.init_executors = self.min_executors
        self.queue = first_not_null(self.queue, None)
        self.executor_environment = first_not_null(
            self.executor_environment, "-")
        return self


class LocalEnvironment(SystemEnvironment):
    '''Class that captures local environment particularities'''
    def __init__(self):
        super(LocalEnvironment, self).__init__()
        if "SPARK_LOCAL_DIRS" in os.environ:
            # This is a slave
            process = psutil.Process().parent().parent().parent()
            self.job_type = LOCAL_EXECUTER_PTYPE
        else:
            # This is the master
            self.configuration = LocalConfiguration()
            self.configuration.complete()
            process = psutil.Process().parent().parent()
            self.job_type = LOCAL_DRIVER_PTYPE
        self.job_id = str(process.pid) + "." + str(process.create_time())
        self.main_id = str(os.getpid()) + "_" + \
            str(process.create_time()).replace(".", "_")

        """Extract required OS ID information """
        if "LOGNAME" in os.environ:
            self.user = os.environ["LOGNAME"]
        elif "USER" in os.environ:
            self.user = os.environ["USER"]

        if "ENVIRONMENT" in os.environ:
            self.environment_type = os.environ["ENVIRONMENT"]
        else:
            self.environment_type = "TEST"
        return
