'''
Created on 26 mars 2018

@author: Tonneau Stephane
'''
# if 'HADOOP_IDENT_STRING' in os.environ:
import os
from .base_utils import first_not_null
from .base_utils import Configuration
from .base_utils import SystemEnvironment

DEFAULTS = {"spark.driver.memory": "1024m", "spark.executor.memory": "1024m",
            "spark.yarn.queue": "default", "spark.dynamicAllocation.minExecutors": "0",
            "spark.dynamicAllocation.maxExecutors": "999999999",
            "spark.executor.instances": "1"}

SPARK_DRIVER_PTYPE = "spark_driver"
SPARK_EXECUTER_PTYPE = "spark_executer"


class SparkConfiguration(Configuration):
    '''Class that encapsulate a Spark Configuration'''

    def __init__(self, driver_memory=None, executer_memory=None, num_executers=None,
                 queue=None, executor_environment=None):
        # parent initialisation
        super(SparkConfiguration, self).__init__(driver_memory, executer_memory, num_executers,
                                                 queue, executor_environment)

    def complete(self, conf):
        '''Fix the Spark configuration information'''

        self.driver_memory = str(
            first_not_null(self.driver_memory, conf.get("spark.driver.memory"),
                           DEFAULTS['spark.driver.memory']))
        self.executer_memory = str(first_not_null(
            self.executer_memory,
            conf.get("spark.executor.memory"),
            DEFAULTS['spark.executor.memory']))
        # if conf.get("spark.master") is not None:
        #    self.spark_master = first_not_null(self.spark_master, str(conf.get("spark.master")))
        # if conf.get("spark.submit.deployMode") is not None:
        #    self.deploy_mode = str(conf.get("spark.submit.deployMode"))
        if conf.get("spark.dynamicAllocation.enabled") == "true":
            self.min_executors = str(
                first_not_null(conf.get("spark.dynamicAllocation.minExecutors"),
                               DEFAULTS["spark.dynamicAllocation.minExecutors"]))
            self.max_executors = str(
                first_not_null(conf.get("spark.dynamicAllocation.maxExecutors"),
                               DEFAULTS["spark.dynamicAllocation.maxExecutors"]))
            self.num_executers = first_not_null(
                self.num_executers, self.min_executors)
        else:
            self.num_executers = str(
                first_not_null(self.num_executers,
                               conf.get("spark.executor.instances"),
                               DEFAULTS["spark.executor.instances"]))
            self.min_executors = None
            self.max_executors = None

        self.num_executers = self.min_executors
        self.queue = str(
            first_not_null(self.queue,
                           conf.get("spark.yarn.queue"),
                           DEFAULTS["spark.yarn.queue"]))
        if conf.get("spark.yarn.isPython") == "true":
            if conf.get("spark.executorEnv.PYTHONPATH") is not None:
                self.executor_environment = first_not_null(self.executor_environment, str(
                    conf.get("spark.executorEnv.PYTHONPATH")))
        return self


class SparkEnvironment(SystemEnvironment):
    '''Class that captures spark environment particularities'''
    def __init__(self):
        super(SparkEnvironment, self).__init__()
        if "HADOOP_IDENT_STRING" in os.environ:
            # from pyspark import SparkContext
            from pyspark import SparkConf
            try:
                self.configuration = SparkConfiguration()
                conf = SparkConf(True)
                self.configuration.complete(conf)
                self.job_type = SPARK_DRIVER_PTYPE
            except Exception:
                # We are in an executor
                self.job_type = SPARK_EXECUTER_PTYPE

        if "SPARK_USER" in os.environ:
            self.user = os.environ["SPARK_USER"]

        if "ENVIRONMENT" in os.environ:
            self.environment_type = os.environ["ENVIRONMENT"]
        else:
            self.environment_type = "TEST"

        # Extract required spark ID information into a dictionary
        if "HADOOP_TOKEN_FILE_LOCATION" in os.environ:
            splited_info = os.environ["HADOOP_TOKEN_FILE_LOCATION"].split("/")
            i = len(splited_info) - 1
            while i >= 0:
                if splited_info[i].startswith("application_"):
                    if self.main_id is None:
                        self.main_id = splited_info[i][12:]
                    self.job_id = splited_info[i]
                elif splited_info[i].startswith("container_"):
                    if self.job_id is None:
                        self.job_id = splited_info[i]
                    self.main_id = splited_info[i][10:]
                i = i - 1
        return
