'''
Created on 10 avril 2018

@author: Tonneau Stephane
'''
from . import pid_entry


class ProcessLog():
    '''Class that allow the logging of the relevant process log information'''

    #AIRFLOW_PTYPE = "airflow"
    #OOZIE_PTYPE = "oozie"

    INITIALIZE_JTYPE = "initialize"
    DOWNLOAD_JTYPE = "download"
    PRETEST_JTYPE = "pretest"
    CONFIGURE_JTYPE = "configure"
    PROCESS_JTYPE = "process"
    POSTTEST_JTYPE = "posttest"
    QUICKLOOK_JTYPE = "quicklook"
    VALIDATE_JTYPE = "validate"
    ARCHIVE_JTYPE = "archive"
    CLEAN_JTYPE = "clean"

    def __init__(self, pid, meta_information, logger):
        self.pid_entry = pid
        self.meta_information = meta_information
        self.logging_methods = logger

    def add_file(self, path_val, context_val=pid_entry.FileInfo.CREATED_CONTEXT,
                 type_val=None, geometry=None,
                 checksum=None, size=None, product_id=None, ogc_urn=None):
        '''add a file information to the log entry'''
        file_info = pid_entry.FileInfo(path_val, context_val, type_val, geometry,
                                       checksum, size, product_id, ogc_urn)
        self.pid_entry.files.append(file_info)
        return self

    def write_log(self):
        '''write the logs in the different logging systems'''
        for key in self.logging_methods:
            key.process_log(self.pid_entry, self.meta_information)
        return self

    def start_logging(self):
        '''write the logs in the different logging systems'''
        self.write_log()
        return self

    def close_logging(self):
        '''close the different logging systems'''
        for key in self.logging_methods:
            key.close()
        return self

    def stop_logging(self, exit_code, message=None):
        '''update the end time and write the logs in the different logging systems'''
        if message is not None:
            self.pid_entry.message = message
        if exit_code is not None:
            self.pid_entry.exit_code = str(exit_code)
        self.pid_entry.finalise()
        self.write_log()
        self.close_logging()
        return self
