'''
Created on 26 mars 2018

@author: Stephane Tonneau

Based on the elasticsearch library :
    to first install with pip install elasticsearch

'''

from . import elasticsearch
from .elasticsearch import Elasticsearch
# from .elasticsearch import helpers
from .base_utils import LoggingMethod

NODENAME = "elasticsearch"
# ESINDEX = "testprobav4"
# ESTYPE = "products4"


def get_elasticsearch(urls):
    '''return the elastic elastic search parameters'''
    if urls is None:
        return None
    return Elasticsearch(
        urls.split(","),
        # sniff before doing anything
        sniff_on_start=False,
        # refresh nodes after a node fails to respond
        sniff_on_connection_fail=False,
        # and also every 60 seconds
        sniffer_timeout=60
    )


class ElasticsearchLogging(LoggingMethod):
    '''Class that encapsulate elasticsearch logging'''

    def __init__(self, configuration=None):
        super(ElasticsearchLogging, self).__init__(configuration)
        es_urls = self.get_config(NODENAME, "hosts")
        if es_urls is None:
            self.logger.error(
                "Missing urls configuration value for ElasticsearchLogging")
        self.esindex = self.get_config(NODENAME, "index")
        if self.esindex is None:
            self.logger.error(
                "Missing index configuration value for ElasticsearchLogging")
        self.estype = self.get_config(NODENAME, "type")
        if self.estype is None:
            self.logger.error(
                "Missing type configuration value for ElasticsearchLogging")
        self.es_connector = get_elasticsearch(es_urls)
        if self.es_connector is None:
            self.logger.error(
                "Elastic search module not configured properly")

    def close(self):
        '''Close the logging instance'''
        pass

    def process_log(self, entry, meta_information):
        '''Method to index a dictionary presentation of the object in elastic search'''
        res = None
        if self.es_connector is not None:
            try:
                doc = entry.to_dict()
                res = self.es_connector.index(
                    index=self.esindex, doc_type=self.estype, id=meta_information.uid, body=doc)
            except:
                self.logger.error(
                    "LOGGING ISSUE : unable to process the logging entry with elastic search")
        return res
