'''
Created on 26 mars 2018

@author: Stephane Tonneau

'''

import os
import datetime
from . import pid_entry
from . import log_entry
from . import base_utils
from .environment_factory import EnvironmentFactory

PROC_INFO_PATH = "procinfodspath"


def fix_for_test():
    '''function to created env variable for testing'''
    os.environ["elasticsearch.index"] = "testprobav4"
    os.environ["elasticsearch.type"] = "products4"
    os.environ[
        "elasticsearch.hosts"] = "es1.vgt.vito.be,es2.vgt.vito.be,es3.vgt.vito.be"
    os.environ[
        "kafka.hosts"] = "epod1.vgt.vito.be:6668,epod17.vgt.vito.be:6668"
    # os.environ["kafka.secure"] = "gssapi"   pid_test
    os.environ["kafka.queue"] = "pid_test2_es"


class LoggingFactory():
    '''Class that encapsulate logging methods'''

    def __init__(self, **logger_info):
        # parent initialisation
        self.logging_methods = []
        configuration = None
        config_path = None
        if logger_info is not None:
            if "classes" in logger_info:
                self.logging_methods = logger_info["classes"]
            if PROC_INFO_PATH in logger_info:
                config_path = logger_info[PROC_INFO_PATH]
            if "env" in logger_info:
                tmp_dict = logger_info["env"]
                for key in tmp_dict:
                    os.environ[key] = tmp_dict[key]
        if PROC_INFO_PATH in os.environ:
            config_path = os.environ[PROC_INFO_PATH]
        if config_path is not None:
            configuration = base_utils.read_config(config_path)
        if not self.logging_methods:
            logger_name = base_utils.get_config(
                configuration, "main", "logger")
            if logger_name is None:
                logger_tab = ("console", "kafka")
                fix_for_test()
            else:
                logger_tab = logger_name.split(",")
            if "file" in logger_tab:
                self.use_file(configuration)
            if "elasticsearch" in logger_tab:
                self.use_elastic(configuration)
            if "console" in logger_tab:
                self.use_console(configuration)
            if "kafka" in logger_tab:
                self.use_kafka(configuration)

    def use_file(self, configuration=None):
        '''declare the filesystem as logging system'''
        from . import logging_file
        self.logging_methods.append(logging_file.FileLogging(configuration))
        return self

    def use_elastic(self, configuration=None):
        '''declare elastic search as logging system'''
        from . import logging_elasticsearch
        self.logging_methods.append(
            logging_elasticsearch.ElasticsearchLogging(configuration))
        return self

    def use_console(self, configuration=None):
        '''declare the console as logging system'''
        from . import logging_console
        self.logging_methods.append(
            logging_console.ConsoleLogging(configuration))
        return self

    def use_kafka(self, configuration=None):
        '''declare the console as logging system'''
        from . import logging_kafka
        self.logging_methods.append(logging_kafka.KafkaLogging(configuration))
        return self

    def get_process_log(self, product_id, product_type, product_date):
        '''return a new ProcessLog object '''
        entry = pid_entry.PidEntry(product_id, product_type, product_date)
        environment = EnvironmentFactory().environment
        entry.configuration = environment.configuration
        entry.job_id = environment.job_id
        entry.job_type = environment.job_type
        entry.user = environment.user
        entry.host = environment.host
        entry.start_time = datetime.datetime.now()
        entry.environment = environment.environment_type
        meta_information = pid_entry.Metainformation()
        meta_information.uid = environment.get_uid()
        return log_entry.ProcessLog(entry, meta_information, self.logging_methods)
