'''
Created on 26 mars 2018

@author: Stephane Tonneau

'''

from .base_utils import LoggingMethod

NODENAME = "file"

# TEMPLATES = ["/tmp_epod/pid/logs/loggingEntry_{0}", "/tmp/loggingEntry_{0}"]
TEMPLATES = ["./loggingEntry_{0}"]


class FileLogging(LoggingMethod):
    '''Class that encapsulate file logging methods'''

    def __init__(self, configuration=None):
        # parent initialisation
        super(FileLogging, self).__init__(configuration)
        es_filename = self.get_config(NODENAME, "filename")
        if es_filename is None:
            self.templates = TEMPLATES
        else:
            self.templates = []
            templates = es_filename.split(",")
            for value in templates:
                self.templates.append(value+"_{0}")
        self.file_position = 0
        self.active_template = 0

    def close(self):
        '''Close the logging instance'''
        pass

    def process_log(self, entry, meta_information):
        '''Method to print a json presentation of the object in a file'''
        doc = entry.to_json()

        filename = self.templates[self.active_template].format(meta_information.uid)
        nbr = len(self.templates)
        while nbr > 0:
            try:
                with open(filename, "w") as text_file:
                    text_file.write(str(doc))
                break
            except Exception:
                self.active_template += 1
                if self.active_template >= len(self.templates):
                    self.active_template = 0
                filename = self.templates[
                    self.active_template].format(meta_information.uid)
                nbr -= 1
        if nbr <= 0:
            self.logger.error(
                "LOGGING ISSUE : unable to process the logging entry to file ")
        self.file_position += 1
