'''
Created on 26 mars 2018

@author: Stephane Tonneau

Based on the elasticsearch library :
    to first install with pip install elasticsearch

'''
import sys
try:
    sys.path.append('/usr/lib64/python2.7/site-packages/')
    import gssapi
except ImportError:
    pass
# Avoiding the import of snappy as VITO has also a snappy
# package from ESA (for other things).  The side effect of
# the ESA package is that it induces a 'too many file open'
# like issue for kafka producer.
sys.modules['snappy'] = None
from kafka import KafkaProducer
from .base_utils import LoggingMethod

# KAFKAHOST = ['epod1.vgt.vito.be:6667','epod17.vgt.vito.be:6667']
# KAFKAQUEUE = "pid_test"
NODENAME = "kafka"


class KafkaLogging(LoggingMethod):
    '''Class that encapsulate kafka logging'''

    def __init__(self, configuration=None):
        super(KafkaLogging, self).__init__(configuration)
        kafka_hosts = self.get_config(NODENAME, "hosts")
        if kafka_hosts is None:
            self.kafka_hosts = None
            self.logger.error(
                "Missing hosts configuration value for KafkaLogging")
        else:
            self.kafka_hosts = kafka_hosts.split(",")
        self.kafka_queue = self.get_config(NODENAME, "queue")
        if self.kafka_queue is None:
            self.logger.error(
                "Missing queue configuration value for KafkaLogging")
        self.secure = self.get_config(NODENAME, "secure")
        try:
            if self.secure is None:
                self.producer = KafkaProducer(
                    bootstrap_servers=self.kafka_hosts, acks='all')
            else:
                self.producer = KafkaProducer(
                    bootstrap_servers=self.kafka_hosts, security_protocol='SASL_PLAINTEXT', sasl_mechanism='GSSAPI', acks='all')
            if self.producer is None:
                self.logger.error(
                    "Kafka logging : kafka producer NOT CREATED !!!")
        except:
            self.logger.error(
                "LOGGING ISSUE : unable to create a kafka producer")
            self.producer = None

    def close(self):
        '''Close the logging instance'''
        if self.producer is not None:
            try:
                self.producer.flush()
                self.producer.close()
            except:
                self.logger.error(
                    "LOGGING ISSUE : unable to close kafka system")
            self.producer = None

    def process_log(self, entry, meta_information):
        '''Method to index a dictionary presentation of the object in elastic search'''
        res = None
        if self.producer is not None:
            try:
                entry.id = meta_information.uid
                future = self.producer.send(
                    self.kafka_queue, str(entry.to_json()))
                res = future.get(timeout=60)
            except:
                self.logger.error(
                    "LOGGING ISSUE : unable to process the logging entry with kafka")
        return res
